
from pgm import widget, render, log
from pgm.layout import window
from pgm.form import rectangle, image, stroke, color, stream, label
import time, sys, Image, gobject

render.init('opengl1')
#log.set_level(log.DEBUG)

# Globals
rotation = 0.
x_offset = 0
y_offset = 0
pressed = False
dragged = False
rect1_back = None

def is_over_surface(drb, button):
    global x_offset, y_offset
    width, height = drb.size
    x, y = drb.position

    over = (button.x > x and button.x < (x + width)) and \
           (button.y > y and button.y < (y + height))
    if over:
        x_offset = button.x - x
        y_offset = button.y - y
    return over

def rotate(rect=None):
    global rotation
    rotation += 0.5
    rect.rotation = (0., 0., rotation)

def motion_notify_cb(event, drb):
    global pressed, dragged, x_offset, y_offset, rect1_back
    if pressed and dragged:
        drb.position = (event.x - x_offset, event.y - y_offset)
        rect1_back.position = (event.x - x_offset - 15, event.y - y_offset - 15)

def button_press_cb(event, drb):
    global pressed, dragged
    if is_over_surface(drb, event):
        print 'Start dragging.'
        dragged = True
        pressed = True

def button_release_cb(event):
    global pressed, dragged
    if dragged:
        print 'Stop dragging.'
        dragged = False
    pressed = False

def key_press_cb(event):
    print event.keyval

def resize_cb(event, win):
    win.regenerate_widgets()

def main(args):
    global rect1_back
    title = 'Drag this video'

    if len(args) != 1:
        print 'Usage: %s stream' % sys.argv[0]
        return 1

    win = window.Window(title=title, color=color.Color((200,200,200,255)),
                        size=(800,600))

    print 'Resolution:', win.screen_resolution
    print 'Size (px): ', win.screen_size
    print 'Size (mm): ', win.screen_size_mm

    rect1_back = rectangle.RoundRectangle(size=(230,180), position=(285,235),
                                          arc=(10,10),
                                          stroke=stroke.Stroke(width=1.5,
                                                color=color.Color(color.BLACK)),
                                          paint=color.Color((50,100,255,255)))
    rect1_back.opacity = 100
    rect1_back.ordering = widget.NEAR
    rect1 = rectangle.Rectangle(size=(200,150), position=(300,250))
    rect1.ordering = widget.NEAR
    rect_bg = rectangle.Rectangle(size=(790,590), position=(5,5))
    rect_bg.opacity = 100
    rect_bg.ordering = widget.NEAR

    video = stream.Stream(location=sys.argv[1])
    video.play()
    rect_bg.media = video
    rect1.media = video

    win.add(rect_bg)
    win.add(rect1_back)
    win.add(rect1)

    lbl = label.Label(size=(800,50), font_size=16, weight=label.WEIGHT_BOLD,
                      text=sys.argv[1], color=color.Color(color.WHITE))
    lbl.ordering = widget.FAR
    win.add(lbl)

    win.sig_exit_event.connect(lambda _: win.main_quit())
    win.sig_button_press_event.connect(button_press_cb, data=rect1)
    win.sig_button_release_event.connect(button_release_cb)
    win.sig_key_press_event.connect(key_press_cb)
    win.sig_motion_notify_event.connect(motion_notify_cb, rect1)
    win.sig_resize_event.connect(resize_cb, win)

    gobject.timeout_add(25, win.update)
    win.main()

if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))
