#include <pgmrender.h>
#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif /* HAVE_CONFIG_H */

#define INCREMENT 0.5f

static PgmRenderBaseWindow *win = NULL;
static PgmRenderDrawable *drb = NULL;
static gboolean pressed = FALSE;
static gboolean dragged = FALSE;
static gint x_offset = 0;
static gint y_offset = 0;
static gfloat increment = INCREMENT;
static gboolean loop = FALSE;
static gboolean fullscreen = FALSE;
static gfloat rotate = 0.0f;

static gboolean is_over_surface (PgmRenderEventButton *button);
static void handle_event (GList *event);

static void
handle_event (GList *event)
{
  while (event)
    {
      PgmRenderEvent *ev = event->data;

      switch (ev->type)
	{
	case PGM_RENDER_DELETE:
	  pgm_render_base_window_quit_loop (win);
	  loop = FALSE;
	  break;
	  
	case PGM_RENDER_CONFIGURE:
	  {
/* 	    PgmRenderEventConfigure *conf = (PgmRenderEventConfigure*) ev; */
/* 	    pgm_render_base_window_set_canvas_size */
/* 	     (win, conf->width, conf->height); */
/* 	    pgm_render_base_window_set_scale (win, conf->width / 800.0f, */
/* 					      conf->height / 600.0f); */
/* 	    pgm_render_base_window_regenerate_drawables (win); */
	    break;
	  }

	case PGM_RENDER_BUTTON_PRESS:
/* 	  pgm_render_base_window_regenerate_drawables (win); */
	  break;

	case PGM_RENDER_KEY_PRESS:
	  pgm_render_base_window_set_size (win, 600, 480);
/* 	  fullscreen = !fullscreen; */
/* 	  if (fullscreen) */
/* 	    pgm_render_base_window_set_fullscreen (win, TRUE, 1024, 768); */
/* 	  else */
/* 	    pgm_render_base_window_set_fullscreen (win, FALSE, 800, 600); */
/* 	  break; */

	default:
	  break;
	}

      g_free (ev);
      event = g_list_next (event);
    }

  g_list_free (event);
}

int
main (int argc, char *argv[])
{
  gint screen_res_width, screen_res_height;
  gint screen_size_width, screen_size_height;
  gint screen_size_width_mm, screen_size_height_mm;
  const gchar *title = "Label";
  GList *event;

  /* Load the OpenGL 1.2 renderer plugin */
  pgm_render_init ("opengl1");

  /* Create the Window */
  win = pgm_render_create_window ();

  pgm_render_base_window_set_scale_mode (win, PGM_RENDER_WINDOW_ADJUST);

  pgm_render_base_window_get_screen_resolution
    (win, &screen_res_width, &screen_res_height);
  pgm_render_base_window_get_screen_size
    (win, &screen_size_width, &screen_size_height);
  pgm_render_base_window_get_screen_size_mm
    (win, &screen_size_width_mm, &screen_size_height_mm);
  g_print ("Resolution: %dx%d\nSize: %dx%d (px)\nSize %dx%d (mm)\n",
	   screen_res_width, screen_res_height,
	   screen_size_width, screen_size_height,
	   screen_size_width_mm, screen_size_height_mm);

  pgm_render_base_window_set_title (win, title, title);
  pgm_render_base_window_set_color (win, 130, 130, 130, 255);
  pgm_render_base_window_set_size (win, 400, 200);
  pgm_render_base_window_set_canvas_size (win, 400, 200);

  drb = pgm_render_create_font ();
  pgm_render_text_set_font (PGM_RENDER_TEXT_IFACE (drb), "Sans",
			    PGM_RENDER_TEXT_WEIGHT_BOLD,
			    PGM_RENDER_TEXT_SLANT_ITALIC, 30);
  pgm_render_layout_set_size (PGM_RENDER_LAYOUT_IFACE (drb), 400, 200);
  pgm_render_layout_set_position (PGM_RENDER_LAYOUT_IFACE (drb), 0, 0);
  pgm_render_base_window_add_drawable (win, drb, PGM_RENDER_DRAWABLE_NEAR);
  pgm_render_text_set_label (PGM_RENDER_TEXT_IFACE (drb),
			     "Pigment");
  pgm_render_text_set_color (PGM_RENDER_TEXT_IFACE (drb), 255, 255, 255, 255);
/*   pgm_render_pipe3d_set_rotation (PGM_RENDER_PIPE3D_IFACE (drb), */
/* 				   5.0f, 45.0f, 0.0f); */

  /* Start the main loop */
  pgm_render_base_window_start_loop (win);
  loop = TRUE;

  while (loop)
    {
      g_usleep (10000);
      pgm_render_base_window_get_events (win, &event);
      handle_event (event);
/*       pgm_render_pipe3d_set_rotation (PGM_RENDER_PIPE3D_IFACE (drb), */
/* 				      0.0f, rotate, 0.0f); */
/*       rotate+=1.0f; */
      pgm_render_base_window_update (win);
    }

  /* Unload the plugin */
  pgm_render_deinit ();

  return 0;
}
