#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include <pgmrender.h>
#include <gst/gst.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#define INCREMENT 0.5f

static PgmRenderBaseWindow *win = NULL;
static PgmRenderDrawable *drb1_back = NULL, *drb1 = NULL,
  *drb2_back = NULL, *drb2 = NULL;
static gfloat increment = INCREMENT;
static gboolean loop = FALSE;
gpointer pixels = NULL;
PgmRenderMediaPixelFormat format;
gint width, height, stride;
GstElement *pipeline, *sink_element, *src_element, *render_sink;

static void handle_event (GList *event);
static void load_image (const gchar *filename);

static void
handle_event (GList *event)
{
  static PgmRenderMediaInterpType type = PGM_RENDER_MEDIA_BILINEAR;

  while (event)
    {
      PgmRenderEvent *ev = event->data;

      switch (ev->type)
	{
	case PGM_RENDER_DELETE:
	  pgm_render_base_window_quit_loop (win);
	  loop = FALSE;
	  break;

	case PGM_RENDER_KEY_PRESS:
	  type = type == PGM_RENDER_MEDIA_BILINEAR ?
	    PGM_RENDER_MEDIA_NEAREST : PGM_RENDER_MEDIA_BILINEAR;
	  pgm_render_media_set_interp_type (PGM_RENDER_MEDIA_IFACE (drb1), type);
	  break;

	case PGM_RENDER_BUTTON_PRESS:
	  gst_element_set_state (pipeline, GST_STATE_PLAYING);
/* 	  g_print ("Click before\n"); */
/* 	  pgm_render_media_set_image (PGM_RENDER_MEDIA_IFACE (drb1), */
/* 				      format, width, height, stride, pixels); */
/* 	  g_print ("Click after\n"); */

/* 	  pgm_render_layout_set_alignment (PGM_RENDER_LAYOUT_IFACE (drb2), */
/* 					   1.0f, 0.5f); */
/* 	  increment = increment == INCREMENT ? 0.0f : INCREMENT; */
	  break;

	default:
	  break;
	}

      g_free (ev);
      event = g_list_next (event);
    }

  g_list_free (event);
}

static void
load_image (const gchar *filename)
{
  GdkPixbuf *image;

  image = gdk_pixbuf_new_from_file (filename, NULL);
  format = gdk_pixbuf_get_has_alpha (image) ?
    PGM_RENDER_MEDIA_RGBA : PGM_RENDER_MEDIA_RGB;
  width = gdk_pixbuf_get_width (image);
  height = gdk_pixbuf_get_height (image);
  stride = gdk_pixbuf_get_rowstride (image);
  pixels = (gpointer) gdk_pixbuf_get_pixels (image);
  g_print ("Colorspace:%s, Size:%dx%d, Stride:%d\n",
	   format == PGM_RENDER_MEDIA_RGBA ? "RGBA" : "RGB",
	   width, height, stride);
}

int
main (int argc, char *argv[])
{
  const gchar *title = "GStreamer video sink";
  GList *event;

  /* Check the command-line */
  if (argc != 2)
    {
      g_print ("Usage: %s stream\n", argv[0]);
      exit (1);
    }

  /* Load the OpenGL 1.2 renderer plugin */
  pgm_render_init ("opengl1");

  /* Init GStreamer */
  gst_init (&argc, &argv);
  gst_plugin_load_file (PGM_GST_VIDEO_SINK_PATH, NULL);

  /* load_image ("examples/pictures/fluendo.png"); */

  /* Create the Window */
  win = pgm_render_create_window ();
  pgm_render_base_window_set_title (win, title, title);
  
  /* Create and set up a Surface in the middle of the context */
/*   drb1_back = pgm_render_create_surface (); */
/*   pgm_render_media_set_color (PGM_RENDER_MEDIA_IFACE (drb1_back), */
/* 			      0, 0, 255, 150); */
/*   pgm_render_layout_set_position (PGM_RENDER_LAYOUT_IFACE (drb1_back), 50, 50); */
/*   pgm_render_layout_set_size (PGM_RENDER_LAYOUT_IFACE (drb1_back), 700, 500); */

  drb1 = pgm_render_create_surface ();
  pgm_render_layout_set_position (PGM_RENDER_LAYOUT_IFACE (drb1), 0, 0);
  pgm_render_layout_set_size (PGM_RENDER_LAYOUT_IFACE (drb1), 800, 600);
  pgm_render_layout_set_alignment (PGM_RENDER_LAYOUT_IFACE (drb1), 0.5f, 0.5f);
  pgm_render_media_set_color (PGM_RENDER_MEDIA_IFACE (drb1),
			      255, 255, 255, 200);

/*   drb2_back = pgm_render_create_surface (); */
/*   pgm_render_media_set_color (PGM_RENDER_MEDIA_IFACE (drb2_back), */
/* 			      0, 255, 0, 150); */
/*   pgm_render_layout_set_position (PGM_RENDER_LAYOUT_IFACE (drb2_back), 25, 25); */
/*   pgm_render_layout_set_size (PGM_RENDER_LAYOUT_IFACE (drb2_back), 500, 200); */
  
/*   drb2 = pgm_render_create_surface (); */
/*   pgm_render_layout_set_position (PGM_RENDER_LAYOUT_IFACE (drb2), 25, 25); */
/*   pgm_render_layout_set_size (PGM_RENDER_LAYOUT_IFACE (drb2), 500, 200); */
/*   pgm_render_layout_set_alignment (PGM_RENDER_LAYOUT_IFACE (drb2), 0.15f, 0.5f); */
/*   pgm_render_media_set_color (PGM_RENDER_MEDIA_IFACE (drb2), */
/* 			      255, 255, 255, 200); */
/*   pgm_render_media_set_image_from_drawable (PGM_RENDER_MEDIA_IFACE (drb2), */
/* 					    drb1); */

  /* Add the drawable */
/*   pgm_render_base_window_add_drawable (win, drb1_back, PGM_RENDER_DRAWABLE_FAR); */
  pgm_render_base_window_add_drawable (win, drb1, PGM_RENDER_DRAWABLE_NEAR);
/*   pgm_render_base_window_add_drawable (win, drb2_back, PGM_RENDER_DRAWABLE_FAR); */
/*   pgm_render_base_window_add_drawable (win, drb2, PGM_RENDER_DRAWABLE_NEAR); */
  
  /* GStreamer pipeline creation */
  pipeline = gst_element_factory_make ("playbin", NULL);
  render_sink = gst_element_factory_make ("pgmrendersink", NULL);
  g_object_set (G_OBJECT (pipeline), "uri", argv[1], NULL);
  g_object_set (G_OBJECT (pipeline), "video-sink", render_sink, NULL);
  g_object_set (G_OBJECT (render_sink), "drawable", drb1, NULL);

  /* Start the main loop */
  pgm_render_base_window_start_loop (win);

  loop = TRUE;
  while (loop)
    {
      g_usleep (20000);
      pgm_render_base_window_get_events (win, &event);
      handle_event (event);
      pgm_render_base_window_update (win);
    }

  gst_deinit ();
  pgm_render_deinit ();

  return 0;
}
